/*
    CFS0.11 filesystem implementation for FUSE
    Copyright (C) 2005-2020 Kajtar Zsolt (Soci/Singular) <soci@c64.rulez.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
#include "file.h"

static File files_head;

File *file_lookup(const resolv_t *at) {
    File *file;
    for (file = files_head.next; file != NULL; file = file->next) {
        if (file->partition == at->partition &&
                file->dir_sector == at->dir_sector &&
                file->dir_offset == at->file_offset) return file;
    }
    return NULL;
}

File *file_open(const resolv_t *at) {
    File *file = file_lookup(at);
    
    if (file == NULL) {
        file = (File *)malloc(sizeof *file);
        if (file == NULL) return NULL;
        file->refcount = 0;
        file->treeroot = at->root;
        file->lastblock = at->root;
        file->lasttree = at->dir_sector;
        file->partition = at->partition;
        file->dir_sector = at->dir_sector;
        file->dir_offset = at->file_offset;
        file->next = files_head.next;
        files_head.next = file;
    }
    file->refcount++;
    return file;
}

void file_close(File *file) {
    File *f;
    file->refcount--;
    if (file->refcount != 0) return;
    for (f = &files_head; f->next != NULL; f = f->next) {
        if (f->next == file) {
            f->next = file->next;
            break;
        }
    }
    free(file);
}
